from batou.component import Component
from batou import UpdateNeeded
import logging


logger = logging.getLogger(__name__)


class VirtualEnv(Component):
    """Manage a virtualenv installation.
    """

    namevar = 'version'
    _clean = False

    # XXX unsure whether this factoring is OK.
    # Depending on the platform and or environment the python executable may
    # not be in the search path and may not even have a predictable path.

    # Alternatively we could have (Unix) environment variables defined in the
    # (batou) environment definition. (Sounds like a good idea.)

    @property
    def python(self):
        """Path to the generated python executable."""
        return 'bin/python{}'.format(self.version)

    def verify(self):
        self.assert_file_is_current(self.python)
        try:
            # If the Python is broken enough, we have to clean it _a lot_
            self.cmd('{} -c "import pkg_resources"'.format(self.python),
                     silent=True)
        except RuntimeError:
            self._clean = True
            raise UpdateNeeded()

    def _detect_virtualenv(self):
        # Prefer the virtualenv of the target version. There are some
        # incompatibilities between Python and virtualenv versions.
        possible_executables = [
            ('virtualenv-{}'.format(self.version), '--no-site-packages'),
            ('virtualenv', '--no-site-packages --python python{}'.format(
                self.version))]
        for executable, arguments in possible_executables:
            try:
                self.cmd('type {}'.format(executable), silent=True)
            except RuntimeError:
                pass
            else:
                break
        else:
            raise RuntimeError('Could not find virtualenv executable')
        return '{} {}'.format(executable, arguments)

    def update(self):
        if self._clean:
            self.cmd('rm -rf bin/ lib/ include/')
        commandline = self._detect_virtualenv()
        target = '.'
        self.cmd('{} {}'.format(commandline, target))


class PIP(Component):

    namevar = 'version'
    version = '1.3'

    def verify(self):
        try:
            result, _ = self.cmd('bin/pip --version')
        except RuntimeError:
            raise UpdateNeeded()
        if not result.startswith('pip {} '.format(self.version)):
            raise UpdateNeeded()

    def update(self):
        self.cmd('bin/pip install --upgrade "pip=={}"'.format(self.version))


class Package(Component):

    namevar = 'package'
    version = None
    check_package_is_module = True
    timeout = None

    pip_install_options = ('--egg', '--force-reinstall')

    def configure(self):
        if self.timeout is None:
            self.timeout = self.environment.timeout

    def verify(self):
        # Is the right version installed according to PIP?
        try:
            self.cmd(
                'bin/python -c "import pkg_resources; '
                'assert pkg_resources.require('
                '\'{}\')[0].version == \'{}\'"'
                .format(self.package, self.version), silent=True)
        except RuntimeError, e:
            logger.debug(e[3])
            raise UpdateNeeded()
        # Is the package usable? Is the package a module?  This might be
        # overspecific - I'm looking for a way to deal with:
        # https://github.com/pypa/pip/issues/3 if a namespace package was not
        # installed cleanly. This only works (currently), when the package name
        # corresponds with what it contains. I.E. it works for zc.buildout but
        # not for distribute, which installs a setuptools package.
        if self.check_package_is_module:
            base_package = self.package.split('.')[0]
            try:
                self.cmd('bin/python -c "import {0};{0}.__file__"'.format(
                    base_package), silent=True)
            except RuntimeError:
                self.pip_install_options += ('-I', '--no-deps')
                raise UpdateNeeded()

    def update(self):
        options = ' '.join(self.pip_install_options)
        self.cmd('bin/pip --timeout={} install {} '
                 '"{}=={}"'.format(
                     self.timeout, options, self.package, self.version))

    @property
    def namevar_for_breadcrumb(self):
        return '{}=={}'.format(self.package, self.version)
